import pytest
from dingus import Dingus
from bunch import Bunch

from jobtriggers import compose_complete_msg


@pytest.mark.usefixtures('prepare')
class TestComposeCompletedJobTrigger():
    @pytest.fixture
    def prepare(self, monkeypatch):
        self.ref_arch = ''
        self.ref_branch = '21'
        self.ref_topic = 'org.fedoraproject.prod.compose.branched.complete'

        self._create_msg(self.ref_arch, self.ref_branch, self.ref_topic)

        self.ref_tasks = ['compose_task1', 'compose_task2']
        self.ref_arches = 'noarch'

        stub_hub = Bunch(config=Bunch(get=0))

        self.helper = compose_complete_msg.ComposeCompletedJobTrigger(stub_hub)
        self.helper.trigger = Dingus()

        compose_complete_msg.config.compose_completed_tasks = self.ref_tasks
        compose_complete_msg.config.valid_arches = self.ref_arches
        compose_complete_msg.config.job_logging = False

    def _create_msg(self, ref_arch, ref_branch, ref_topic):
        self.ref_message = Bunch(body='{"i": 1,\
                                  "msg": {"arch": "%s",\
                                          "branch": "%s",\
                                          "log": "done"},\
                                  "topic": "%s"}' %
                                  (ref_arch, ref_branch, ref_topic))

    def test_consume(self):
        self.helper.consume(self.ref_message)

        trigger_calls = self.helper.trigger.trigger_job.calls()

        assert len(trigger_calls) == 2

        assert trigger_calls[0][1] == (self.ref_branch, compose_complete_msg.ITEM_TYPE, self.ref_tasks[0], self.ref_arches)
        assert trigger_calls[1][1] == (self.ref_branch, compose_complete_msg.ITEM_TYPE, self.ref_tasks[1], self.ref_arches)

    def test_consume_secondary_arch(self):
        ref_arch = 's390'
        self._create_msg(ref_arch, self.ref_branch, self.ref_topic)

        self.helper.consume(self.ref_message)

        trigger_calls = self.helper.trigger.trigger_job.calls()

        assert len(trigger_calls) == 0

    def test_consume_incorrect_topic(self):
        ref_topic = 'fake_topic'
        self._create_msg(self.ref_arch, self.ref_branch, ref_topic)

        self.helper.consume(self.ref_message)

        trigger_calls = self.helper.trigger.trigger_job.calls()

        assert len(trigger_calls) == 0

    def test_consume_allowed_topics(self):
        ref_topics = ['org.fedoraproject.prod.compose.branched.complete', 'org.fedoraproject.prod.compose.rawhide.complete']

        for ref_topic in ref_topics:
            self._create_msg(self.ref_arch, self.ref_branch, ref_topic)

            self.helper.consume(self.ref_message)

            trigger_calls = self.helper.trigger.trigger_job.calls()

        assert len(trigger_calls) == 4
